@@ -1,6 +1,6 @@
 """Agent Resource functions."""
 
-# Copyright 2022 Google LLC
+# Copyright 2023 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -237,31 +237,38 @@ class Agents(scrapi_base.ScrapiBase):
           display_name: Human readable display name for the CX agent
           gcp_region: GCP region to create CX agent. Defaults to 'global'
           obj: (Optional) Agent object to create new agent from
+            Refer to `builders.agents.AgentBuilder` to build one.
 
         Returns:
           The newly created CX Agent resource object.
         """
 
+        parent = f"projects/{project_id}/locations/{gcp_region}"
+
         if obj:
-            agent = obj
-            parent = f"projects/{project_id}/locations/{gcp_region}"
+            agent_obj = obj
+            agent_obj.name = ""
 
         else:
-            agent = types.agent.Agent()
-            parent = f"projects/{project_id}/locations/{gcp_region}"
-            agent.display_name = display_name
-            agent.default_language_code = "en"
-            agent.time_zone = "America/Chicago"
+            if not display_name:
+                raise ValueError(
+                    "At least display_name or obj should be specified."
+                )
+            agent_obj = types.Agent(
+                display_name=display_name,
+                default_language_code="en",
+                time_zone="America/Chicago"
+            )
 
             # set optional args as agent attributes
             for key, value in kwargs.items():
-                setattr(agent, key, value)
+                setattr(agent_obj, key, value)
 
         client_options = self._set_region(parent)
         client = services.agents.AgentsClient(
             credentials=self.creds, client_options=client_options
         )
-        response = client.create_agent(parent=parent, agent=agent)
+        response = client.create_agent(parent=parent, agent=agent_obj)
 
         return response
 
